@@ -12,8 +12,8 @@
  *** modified after it is created, only the contained feature_recorders are modified.
  ****************************************************************/
 
-const string feature_recorder_set::ALERT_RECORDER_NAME = "alerts";
-const string feature_recorder_set::DISABLED_RECORDER_NAME = "disabled";
+const std::string feature_recorder_set::ALERT_RECORDER_NAME = "alerts";
+const std::string feature_recorder_set::DISABLED_RECORDER_NAME = "disabled";
 
 /* Create an empty recorder */
 feature_recorder_set::feature_recorder_set(uint32_t flags_):flags(flags_),seen_set(),input_fname(),
@@ -35,17 +35,15 @@ feature_recorder_set::feature_recorder_set(uint32_t flags_):flags(flags_),seen_s
  */
 void feature_recorder_set::init(const feature_file_names_t &feature_files,
                                 const std::string &input_fname_,
-                                const std::string &outdir_,
-                                const histograms_t *histogram_defs_)
+                                const std::string &outdir_)
 {
     input_fname    = input_fname_;
     outdir         = outdir_;
-    histogram_defs = histogram_defs_;
 
     create_name(feature_recorder_set::ALERT_RECORDER_NAME,false); // make the alert recorder
 
     /* Create the requested feature files */
-    for(set<string>::const_iterator it=feature_files.begin();it!=feature_files.end();it++){
+    for(std::set<std::string>::const_iterator it=feature_files.begin();it!=feature_files.end();it++){
         create_name(*it,flags & CREATE_STOP_LIST_RECORDERS);
     }
 }
@@ -65,7 +63,7 @@ void feature_recorder_set::close_all()
 }
 
 
-bool feature_recorder_set::has_name(string name) const
+bool feature_recorder_set::has_name(std::string name) const
 {
     return frm.find(name) != frm.end();
 }
@@ -108,7 +106,7 @@ void feature_recorder_set::create_name(const std::string &name,bool create_stop_
 
     frm[name] = fr;
     if(create_stop_file){
-        string name_stopped = name+"_stopped";
+        std::string name_stopped = name+"_stopped";
         
         fr_stopped = create_name_factory(outdir,input_fname,name_stopped);
         fr->set_stop_list_recorder(fr_stopped);
@@ -137,7 +135,7 @@ bool feature_recorder_set::check_previously_processed(const uint8_t *buf,size_t
     return seen_set.check_for_presence_and_insert(md5);
 }
 
-void feature_recorder_set::add_stats(string bucket,double seconds)
+void feature_recorder_set::add_stats(const std::string &bucket,double seconds)
 {
     cppmutex::lock lock(map_lock);
     struct pstats &p = scanner_stats[bucket]; // get the location of the stats
@@ -168,42 +166,19 @@ void feature_recorder_set::dump_name_count_stats(dfxml_writer &writer)
 
 
 static const int LINE_LEN = 80;         // keep track of where we are on the line
-void feature_recorder_set::process_histograms(feature_recorder_set::xml_notifier_t xml_error_notifier)
+void feature_recorder_set::add_histogram(const histogram_def &def)
 {
-    if(histogram_defs==0) return;       // no defs!
+    feature_recorder *fr = get_name(def.feature);
+    if(fr) fr->add_histogram(def);
+}
 
-    int pos  = 0;                       // for generating \n when printing
-    bool need_nl = false;
-
-    /* Loop through all the histograms */
-    for(histograms_t::const_iterator it = histogram_defs->begin();it!=histogram_defs->end();it++){
-        std::string msg = string(" ") + (*it).feature + " " + (*it).suffix + "...";
-        if(msg.size() + pos > (unsigned) LINE_LEN){
-            std::cout << "\n";
-            pos = 0;
-            need_nl = false;
-        }
-        std::cout << msg;
-        std::cout.flush();
-        pos += msg.size();
-        need_nl = true;
-        if(has_name((*it).feature)){
-            feature_recorder *fr = get_name((*it).feature);
-            try {
-                fr->make_histogram((*it),0);
-            }
-            catch (const std::exception &e) {
-                std::cerr << "ERROR: " ;
-                std::cerr.flush();
-                std::cerr << e.what() << " computing histogram " << (*it).feature << "\n";
-                if(xml_error_notifier){
-                    std::string error = std::string("<error function='phase3' histogram='")
-                        + (*it).feature + std::string("</error>");
-                    (*xml_error_notifier)(error);
-                }
-            }
-        }
+void feature_recorder_set::dump_histograms(void *user,feature_recorder::dump_callback_t cb,
+                                           feature_recorder_set::xml_notifier_t xml_error_notifier)
+{
+    /* Ask each feature recorder to dump its histograms */
+    for(feature_recorder_map::const_iterator it = frm.begin(); it!=frm.end(); it++){
+        feature_recorder *fr = it->second;
+        fr->dump_histograms(user,cb,xml_error_notifier);
     }
-    if (need_nl) std::cout << "\n";
 }
 
