@@ -23,18 +23,6 @@
 #include "../dfxml/src/hash_t.h"
 
 
-//namespace std {
-//    namespace tr1 {
-//        template<>
-//        struct hash<md5_t> {
-//            size_t operator()(const md5_t &key) const {
-//                return *(size_t *)(key.final());
-//            }
-//        };
-//    }
-//}
-//#endif
-
 uint32_t scanner_def::max_depth = 7;            // max recursion depth
 uint32_t scanner_def::max_ngram = 10;            // max recursion depth
 static int debug;                               // local debug variable
@@ -43,6 +31,17 @@ static cppmutex max_depth_seenM;
 bool be13::plugin::dup_data_alerts = false; // by default, is disabled
 uint64_t be13::plugin::dup_data_encountered = 0; // amount that was not processed
 
+class scanner_command {
+public:
+    enum command_t {DISABLE_ALL=0,ENABLE_ALL,DISABLE,ENABLE};
+    scanner_command(const scanner_command &sc):command(sc.command),name(sc.name){};
+    scanner_command(scanner_command::command_t c,const std::string &n):command(c),name(n){};
+    command_t command;
+    std::string name;
+};
+static std::vector<scanner_command> scanner_commands;
+bool scanner_commands_processed = false;
+
 /****************************************************************
  *** misc support
  ****************************************************************/
@@ -94,7 +93,7 @@ public:
     packet_callback_t *callback;
 };
 
-typedef vector<packet_plugin_info> packet_plugin_info_vector_t;
+typedef std::vector<packet_plugin_info> packet_plugin_info_vector_t;
 packet_plugin_info_vector_t  packet_handlers;   // pcap callback handlers
 
 
@@ -149,8 +148,6 @@ void be13::plugin::set_scanner_enabled_all(bool enable)
  * The histogram should be done in the plug-in
  */
 
-//static histograms_t histograms;
-
 /****************************************************************
  *** scanner plugin loading
  ****************************************************************/
@@ -165,7 +162,7 @@ void be13::plugin::set_scanner_enabled_all(bool enable)
  * This is called before scanners are enabled or disabled, so the pcap handlers
  * need to be set afterwards
  */
-void be13::plugin::load_scanner(scanner_t scanner,scanner_info::scanner_config &sc)
+void be13::plugin::load_scanner(scanner_t scanner,const scanner_info::scanner_config &sc)
 {
     /* If scanner is already loaded, return */
     for(scanner_vector::const_iterator it = current_scanners.begin();it!=current_scanners.end();it++){
@@ -202,28 +199,21 @@ void be13::plugin::load_scanner(scanner_t scanner,scanner_info::scanner_config &
     (*scanner)(sp,rcb);                  // phase 0
     
     sd->enabled      = !(sd->info.flags & scanner_info::SCANNER_DISABLED);
-
-    // Catch histograms and add as a current scanner for all scanners,
-    // as a scanner may be enabled after it is loaded.
-    for(histograms_t::const_iterator it = sd->info.histogram_defs.begin();
-        it != sd->info.histogram_defs.end(); it++){
-        sc.histograms.insert((*it));
-    }
     current_scanners.push_back(sd);
 }
 
-void be13::plugin::load_scanner_file(string fn,scanner_info::scanner_config &sc)
+void be13::plugin::load_scanner_file(std::string fn,const scanner_info::scanner_config &sc)
 {
     /* Figure out the function name */
     size_t extloc = fn.rfind('.');
-    if(extloc==string::npos){
+    if(extloc==std::string::npos){
         errx(1,"Cannot find '.' in %s",fn.c_str());
     }
-    string func_name = fn.substr(0,extloc);
+    std::string func_name = fn.substr(0,extloc);
     size_t slashloc = func_name.rfind('/');
-    if(slashloc!=string::npos) func_name = func_name.substr(slashloc+1);
+    if(slashloc!=std::string::npos) func_name = func_name.substr(slashloc+1);
     slashloc = func_name.rfind('\\');
-    if(slashloc!=string::npos) func_name = func_name.substr(slashloc+1);
+    if(slashloc!=std::string::npos) func_name = func_name.substr(slashloc+1);
 
     std::cout << "Loading: " << fn << " (" << func_name << ")\n";
     scanner_t *scanner = 0;
@@ -252,14 +242,14 @@ void be13::plugin::load_scanner_file(string fn,scanner_info::scanner_config &sc)
     load_scanner(*scanner,sc);
 }
 
-void be13::plugin::load_scanners(scanner_t * const *scanners,scanner_info::scanner_config &sc)
+void be13::plugin::load_scanners(scanner_t * const *scanners,const scanner_info::scanner_config &sc)
 {
     for(int i=0;scanners[i];i++){
         load_scanner(scanners[i],sc);
     }
 }
 
-void be13::plugin::load_scanner_directory(const string &dirname,scanner_info::scanner_config &sc )
+void be13::plugin::load_scanner_directory(const std::string &dirname,const scanner_info::scanner_config &sc )
 {
     DIR *dirp = opendir(dirname.c_str());
     if(dirp==0){
@@ -267,11 +257,11 @@ void be13::plugin::load_scanner_directory(const string &dirname,scanner_info::sc
     }
     struct dirent *dp;
     while ((dp = readdir(dirp)) != NULL){
-        string fname = dp->d_name;
+        std::string fname = dp->d_name;
         if(fname.substr(0,5)=="scan_" || fname.substr(0,5)=="SCAN_"){
             size_t extloc = fname.rfind('.');
-            if(extloc==string::npos) continue; // no '.'
-            string ext = fname.substr(extloc+1);
+            if(extloc==std::string::npos) continue; // no '.'
+            std::string ext = fname.substr(extloc+1);
 #ifdef WIN32
             if(ext!="DLL") continue;    // not a DLL
 #else
@@ -283,14 +273,13 @@ void be13::plugin::load_scanner_directory(const string &dirname,scanner_info::sc
 }
 
 void be13::plugin::load_scanner_directories(const std::vector<std::string> &dirnames,
-                                            scanner_info::scanner_config &sc)
+                                            const scanner_info::scanner_config &sc)
 {
     for(std::vector<std::string>::const_iterator it = dirnames.begin();it!=dirnames.end();it++){
         load_scanner_directory(*it,sc);
     }
 }
 
-
 void be13::plugin::load_scanner_packet_handlers()
 {
     for(scanner_vector::const_iterator it = current_scanners.begin(); it!=current_scanners.end(); it++){
@@ -349,43 +338,58 @@ bool be13::plugin::find_scanner_enabled()
 }
 
 
+void be13::plugin::add_enabled_scanner_histograms_to_feature_recorder_set(feature_recorder_set &fs)
+{
+    for(scanner_vector::const_iterator it = current_scanners.begin(); it!=current_scanners.end(); it++){
+        if((*it)->enabled){
+            const scanner_def *sd = (*it);
+            for(histogram_defs_t::const_iterator i2 = sd->info.histogram_defs.begin();
+                i2 != sd->info.histogram_defs.end(); i2++){
+                fs.add_histogram((*i2));
+            }
+        }
+    }
+}
+
+void be13::plugin::scanners_init(feature_recorder_set &fs)
+{
+    assert(scanner_commands_processed==true);
+    message_enabled_scanners(scanner_params::PHASE_INIT,fs); // tell all enabled scanners to init
+}
+
+
 
 /****************************************************************
  *** Scanner Commands (which one is enabled or disabled)
  ****************************************************************/
-class scanner_command {
-public:
-    enum command_t {DISABLE_ALL=0,ENABLE_ALL,DISABLE,ENABLE};
-    scanner_command(const scanner_command &sc):command(sc.command),name(sc.name){};
-    scanner_command(scanner_command::command_t c,const string &n):command(c),name(n){};
-    command_t command;
-    string name;
-};
-static vector<scanner_command> scanner_commands;
 
 void be13::plugin::scanners_disable_all()
 {
-    scanner_commands.push_back(scanner_command(scanner_command::DISABLE_ALL,string("")));
+    assert(scanner_commands_processed==false);
+    scanner_commands.push_back(scanner_command(scanner_command::DISABLE_ALL,std::string("")));
 }
 
 void be13::plugin::scanners_enable_all()
 {
-    scanner_commands.push_back(scanner_command(scanner_command::ENABLE_ALL,string("")));
+    assert(scanner_commands_processed==false);
+    scanner_commands.push_back(scanner_command(scanner_command::ENABLE_ALL,std::string("")));
 }
 
 void be13::plugin::scanners_enable(const std::string &name)
 {
+    assert(scanner_commands_processed==false);
     scanner_commands.push_back(scanner_command(scanner_command::ENABLE,name));
 }
 
 void be13::plugin::scanners_disable(const std::string &name)
 {
+    assert(scanner_commands_processed==false);
     scanner_commands.push_back(scanner_command(scanner_command::DISABLE,name));
 }
 
 void be13::plugin::scanners_process_enable_disable_commands()
 {
-    for(vector<scanner_command>::const_iterator it=scanner_commands.begin();
+    for(std::vector<scanner_command>::const_iterator it=scanner_commands.begin();
         it!=scanner_commands.end();it++){
         switch((*it).command){
         case scanner_command::ENABLE_ALL:  set_scanner_enabled_all(true);break;
@@ -395,20 +399,17 @@ void be13::plugin::scanners_process_enable_disable_commands()
         }
     }
     load_scanner_packet_handlers();     // can't do until enable/disable commands are run
+    scanner_commands_processed = true;
 }
 
 
-void be13::plugin::scanners_init(feature_recorder_set &fs)
-{
-    message_enabled_scanners(scanner_params::PHASE_INIT,fs); // tell all enabled scanners to init
-}
-
 /****************************************************************
  *** PHASE_SHUTDOWN (formerly phase 2): shut down the scanners
  ****************************************************************/
 
 void be13::plugin::phase_shutdown(feature_recorder_set &fs,std::stringstream *sxml)
 {
+    assert(scanner_commands_processed==true);
     for(scanner_vector::iterator it = current_scanners.begin();it!=current_scanners.end();it++){
         if((*it)->enabled){
             const sbuf_t sbuf; // empty sbuf
@@ -508,7 +509,7 @@ void be13::plugin::info_scanners(bool detailed_info,
                                  scanner_t * const *scanners_builtin,
                                  const char enable_opt,const char disable_opt)
 {
-    scanner_info::scanner_config empty_config;
+    const scanner_info::scanner_config empty_config;
 
     load_scanners(scanners_builtin,empty_config);
     std::cout << "\n";
@@ -524,7 +525,7 @@ void be13::plugin::info_scanners(bool detailed_info,
             if ((*it)->info.url.size()) std::cout << "URL: " << (*it)->info.url << "\n";
             if ((*it)->info.scanner_version.size()) std::cout << "Scanner Version: " << (*it)->info.scanner_version << "\n";
             std::cout << "Feature Names: ";
-            for(set<string>::const_iterator i2 = (*it)->info.feature_names.begin();
+            for(std::set<std::string>::const_iterator i2 = (*it)->info.feature_names.begin();
                 i2 != (*it)->info.feature_names.end();
                 i2++){
                 std::cout << *i2 << " ";
@@ -655,14 +656,14 @@ void be13::plugin::process_sbuf(const class scanner_params &sp)
 
             /* Compute the effective path for stats */
             bool inname=false;
-            string epath;
-            for(string::const_iterator cc=sp.sbuf.pos0.path.begin();cc!=sp.sbuf.pos0.path.end();cc++){
+            std::string epath;
+            for(std::string::const_iterator cc=sp.sbuf.pos0.path.begin();cc!=sp.sbuf.pos0.path.end();cc++){
                 if(isupper(*cc)) inname=true;
                 if(inname) epath.push_back(toupper(*cc));
                 if(*cc=='-') inname=false;
             }
             if(epath.size()>0) epath.push_back('-');
-            for(string::const_iterator cc=name.begin();cc!=name.end();cc++){
+            for(std::string::const_iterator cc=name.begin();cc!=name.end();cc++){
                 epath.push_back(toupper(*cc));
             }
 
@@ -674,13 +675,13 @@ void be13::plugin::process_sbuf(const class scanner_params &sp)
             {
                 aftimer t;
                 if(debug & DEBUG_PRINT_STEPS){
-                    cerr << "sbuf.pos0=" << sp.sbuf.pos0 << " calling scanner " << name << "\n";
+                    std::cerr << "sbuf.pos0=" << sp.sbuf.pos0 << " calling scanner " << name << "\n";
                 }
                 t.start();
                 ((*it)->scanner)(sp,rcb);
                 t.stop();
                 if(debug & DEBUG_PRINT_STEPS){
-                    cerr << "sbuf.pos0=" << sp.sbuf.pos0 << " scanner "
+                    std::cerr << "sbuf.pos0=" << sp.sbuf.pos0 << " scanner "
                          << name << " t=" << t.elapsed_seconds() << "\n";
                 }
                 sp.fs.add_stats(epath,t.elapsed_seconds());
@@ -728,7 +729,7 @@ void be13::plugin::get_scanner_feature_file_names(feature_file_names_t &feature_
 {
     for(scanner_vector::const_iterator it=current_scanners.begin();it!=current_scanners.end();it++){
         if((*it)->enabled){
-            for(set<string>::const_iterator fi=(*it)->info.feature_names.begin();
+            for(std::set<std::string>::const_iterator fi=(*it)->info.feature_names.begin();
                 fi!=(*it)->info.feature_names.end();
                 fi++){
                 feature_file_names.insert(*fi);
