@@ -30,8 +30,6 @@
 #define __STDC_FORMAT_MACROS
 #endif
 
-//using namespace std;
-
 #include <stdlib.h>
 #include <inttypes.h>
 #include <string>
@@ -61,13 +59,6 @@
  * in a 64-bit number.  
  */
 
-inline int stoi(std::string str){
-    std::istringstream ss(str);
-    int val(0);
-    ss >> val;
-    return val;
-}
-
 inline int64_t stoi64(std::string str){
     int64_t val(0);
     std::istringstream ss(str);
@@ -188,22 +179,17 @@ inline bool operator ==(const class pos0_t & pos0,const class pos0_t &pos1) {
 
 /**
  * \class managed_malloc Like new[], but it automatically gets freed when the object is dropped.
+ * throws std::bad_alloc if no memory.
  */
-template < class Type > class managed_malloc {
-    class not_impl: public std::exception {
-        virtual const char *what() const throw() {
-            return "managed_malloc assignment is not implemented.";
-        }
-    };
-    managed_malloc &operator=(const managed_malloc &that) {
-        throw new not_impl();
-    }
-    managed_malloc(const managed_malloc &that):buf(0){
-        throw new not_impl();
-    }
+template < class TYPE > class managed_malloc {
+    // default construction, copy construction and assignment are meaningless
+    // and not implemented
+    managed_malloc& operator=(const managed_malloc&); 
+    managed_malloc(const managed_malloc&);
+    managed_malloc();
 public:
-    Type *buf;
-    managed_malloc(size_t bytes):buf(new Type[bytes]){ }
+    TYPE *buf;
+    managed_malloc(size_t bytes):buf(new TYPE[bytes]){ }
     ~managed_malloc(){
         if(buf) delete []buf;
     }
@@ -233,12 +219,6 @@ public:
  * subset will now point to unallocated memory.)
  */
 class sbuf_t {
-private:
-    class not_impl: public std::exception {
-        virtual const char *what() const throw() {
-            return "sbuf_t assignment is not implemented.";
-        }
-    };
 private:
     /* The private structures keep track of memory management */
     int    fd;                          /* file this came from if mmapped file */
@@ -270,9 +250,8 @@ public:
     
 private:
     void release();                     // release allocated storage
-    sbuf_t &operator=(const sbuf_t &that) {
-        throw new not_impl();
-    }
+    // default assignment not implemented
+    sbuf_t &operator=(const sbuf_t &that);
 public:
     /** Make an empty sbuf.
         It's used for situations where an sbuf is needed but not referenced */
@@ -379,7 +358,7 @@ public:
     }
 
     virtual ~sbuf_t(){
-#ifdef SBUF_TRACK
+#if defined(SBUF_TRACK) && defined(HAVE___SYNC_ADD_AND_FETCH)
         assert(__sync_fetch_and_add(&children,0)==0);
 #endif
         if(parent) parent->del_child(*this);
@@ -387,9 +366,13 @@ public:
     }
 
     /* Allocate a sbuf from a file mapped into memory */
-    static sbuf_t *map_file(const std::string &fname,const pos0_t &pos0); 
-    static sbuf_t *map_file(const std::string &fname,const pos0_t &pos0,int fd); 
-    static std::string U10001C;         // delimeter character in bulk_extractor
+    static sbuf_t *map_file(const std::string &fname); 
+    static sbuf_t *map_file(const std::string &fname,int fd); // if file is already opened
+    static const std::string U10001C;         // default delimeter character in bulk_extractor
+    static std::string map_file_delimiter; // character placed
+    static void set_map_file_delimiter(const std::string &new_delim){
+        map_file_delimiter = new_delim;
+    }
 
     /* Properties */
     size_t size() const {return bufsize;} // return the number of bytes
@@ -397,14 +380,14 @@ public:
 
     const sbuf_t *highest_parent() const; // returns the parent of the parent...
     void add_child(const sbuf_t &child) const {
+#if defined(HAVE___SYNC_ADD_AND_FETCH) && defined(SBUF_TRACK)
         __sync_fetch_and_add(&children,1);
-#ifdef SBUF_TRACK
         std::cerr << "add_child(" << this << ")="<<children << "\n";
 #endif
     }
     void del_child(const sbuf_t &child) const {
+#if defined(HAVE___SYNC_ADD_AND_FETCH) && defined(SBUF_TRACK)
         __sync_fetch_and_add(&children,-1);
-#ifdef SBUF_TRACK
         std::cerr << "del_child(" << this << ")="<<children << "\n";
         assert(__sync_fetch_and_add(&children,0)>=0);
 #endif
@@ -567,6 +550,7 @@ public:
      * Find the next occurance of a char* string in the buffer
      * starting at a give point.
      * Return offset or -1 if there is none to find.
+     * This would benefit from a boyer-Moore implementation
      */
     ssize_t find(const char *str,size_t start) const {
         if(str[0]==0) return -1;        // nothing to search for
@@ -590,10 +574,10 @@ public:
 
     // Return a pointer to a structure contained within the sbuf if there is
     // room, otherwise return a null pointer.
-    template<class Type>
-    const Type * get_struct_ptr(uint32_t pos) const {
-        if (pos + sizeof(Type) <= bufsize) {
-            return reinterpret_cast<const Type *> (buf+pos);
+    template<class TYPE>
+    const TYPE * get_struct_ptr(uint32_t pos) const {
+        if (pos + sizeof(TYPE) <= bufsize) {
+            return reinterpret_cast<const TYPE *> (buf+pos);
         }
         return NULL;
     }
