@@ -29,6 +29,7 @@ where encoding, if present, is 0 for raw, 1 for NTFS compressed.
 
 """
 import sys
+import re
 from sys import stderr
 from subprocess import Popen,PIPE
 import base64
@@ -40,6 +41,9 @@ __version__ = "1.0.1"
 
 tsk_virtual_filenames = set(['$FAT1','$FAT2'])
 
+XMLNS_DFXML = "http://www.forensicswiki.org/wiki/Category:Digital_Forensics_XML"
+XMLNS_DELTA = "http://www.forensicswiki.org/wiki/Forensic_Disk_Differencing"
+
 def isone(x):
     """Return true if something is one (number or string)"""
     try:
@@ -82,7 +86,6 @@ def parse_iso8601(ts):
     raise RuntimeError("parse_iso8601: ISO8601 format {} not recognized".format(ts))
 
 
-import re
 rx_iso8601 = re.compile("(\d\d\d\d)-(\d\d)-(\d\d)[T ](\d\d):(\d\d):(\d\d)(\.\d+)?(Z|[-+]\d\d:?\d\d)?")
 def iso8601Tdatetime(s):
     """SLG's conversion of ISO8601 to datetime"""
@@ -747,8 +750,10 @@ class fileobject:
     def content_for_run(self,run=None,imagefile=None):
         """ Returns the content for a specific run. This is a convenience feature
         which does not touch the file object if an imagefile is provided."""
-        if imagefile==None: imagefile=self.imagefile
-        if run.len== -1:
+        if imagefile is None: imagefile=self.imagefile
+        if run is None: raise ValueError("content_for_run called without a 'run' argument.")
+
+        if run.len == -1:
             return chr(0) * run.len
         elif hasattr(run,'fill'):
             return chr(run.fill) * run.len
@@ -940,6 +945,11 @@ register_sax_tag(imageobject_sax,'imagesize')
 register_sax_tag(imageobject_sax,'image_filename')
 
 
+class creatorobject_sax(saxobject):
+    """A class that represents the <creator> section of a DFXML file"""
+for tag in ['creator','program','version']:
+    register_sax_tag(creatorobject_sax,tag)
+
 ################################################################
 
 ################################################################
@@ -975,9 +985,11 @@ class xml_reader:
         if self.cdata != None:
             self.cdata += data
 
-    def process_xml_stream(self,xml_stream,callback):
+    def process_xml_stream(self,xml_stream,callback,preserve_fis=False):
         "Run the reader on a given XML input stream"
         self.callback = callback
+        self.preserve_fis = preserve_fis
+        self.fi_history = []
         import xml.parsers.expat
         p = xml.parsers.expat.ParserCreate()
         p.StartElementHandler  = self._start_element
@@ -1107,7 +1119,7 @@ class regxml_reader(xml_reader):
         """
         The callback is invoked for each stack-popping operation, except the root.
         """
-        #TODO sanity-check the objectstack
+        # TODO sanity-check the objectstack
         if name in ["msregistry","hive"]:
             pass
         elif name in ["key","node"]:
@@ -1191,6 +1203,8 @@ class fileobject_reader(xml_reader):
             return
         if name=="fileobject":
             self.callback(self.fileobject)
+            if self.preserve_fis:
+                self.fi_history.append(self.fileobject)
             self.fileobject = None
             return
         if name=='hashdigest' and len(self.tagstack)>0:
@@ -1213,6 +1227,7 @@ class fileobject_reader(xml_reader):
             self.imageobject._tags['image_filename'] = self.cdata
 
 class volumeobject_reader(xml_reader):
+    """Reads just the <volume> section of a DFXML file"""
     def __init__(self):
         self.volumeobject = False
         xml_reader.__init__(self)
@@ -1251,6 +1266,37 @@ class volumeobject_reader(xml_reader):
         return
 
 
+class FinishedReadingCreator(Exception):
+    """Class to indicate that creator object has been read"""
+
+class creatorobject_reader(xml_reader):
+    """Reads the <creator> section of a DFXML file"""
+    def __init__(self):
+        self.creatorobject = False
+        xml_reader.__init__(self)
+
+    def _start_element(self, name, attrs):
+        """ Handles the start of an element for the XPAT scanner"""
+        self.tagstack.append(name)
+        if name=="creator":
+            self.creatorobject = creatorobject_sax()
+            return
+        if self.creatorobject:
+            self.cdata = ""     # capture cdata for creatorobject
+
+    def _end_element(self, name):
+        """Handles the end of an eleement for the XPAT scanner"""
+        assert(self.tagstack.pop()==name)
+        if name=="creator":
+            self.callback(self.creatorobject)
+            self.creatorobject = None
+            raise FinishedReadingCreator("Done")
+        if self.tagstack[-1]=='creator' and self.creatorobject: # in the creator
+            self.creatorobject._tags[name] = self.cdata
+            self.cdata = None
+            return
+
+
 def combine_runs(runs):
     """Given an array of bytrun elements, combine the runs and return a new array."""
     if runs==[]: return []
@@ -1353,15 +1399,88 @@ class extentdb:
         return filter(lambda x:not self.intersects_sector(x),self.sectors_for_run(run))
 
 
-def read_dfxml(xmlfile=None,imagefile=None,flags=0,callback=None):
+def read_dfxml(xmlfile=None,imagefile=None,flags=0,callback=None,preserve_fis=False):
     """Processes an image using expat, calling a callback for every file object encountered.
     If xmlfile is provided, use that as the xmlfile, otherwise runs fiwalk."""
     if not callback:
         raise ValueError("callback must be specified")
     r = fileobject_reader(imagefile=imagefile,flags=flags)
-    r.process_xml_stream(xmlfile,callback)
+    r.process_xml_stream(xmlfile,callback,preserve_fis)
     return r
 
+def iter_dfxml(xmlfile, preserve_elements=False, imagefile=None):
+    """Returns an interator that yields fileobjects from a DFXML file.
+    
+    @param preserve_elements
+    Yielded fileobjects can also retain the xml.etree.ElementTree.Element,
+    the fileobject's source XML as a manipulable object.
+    Pass preserve_elements=True to get fi.xml_element.
+    NOTE: Retaining Elements is quite memory-intensive.  Creating a MAC
+    timeline from DFXML of the "CFREDS Hacking" image (a 34MB XML file)
+    using demo_mac_timeline_iter.py maxed at 65MB of RAM without
+    preserve_elements, and about 650MB with.  
+    
+    This function might be extended in the future to call Fiwalk (and
+    thus become what fileobjects_iter was supposed to be).
+
+    Note that to serialize the fileobjects to strings, you may wish to
+    use the ET_tostring wrapper function in this module.  The
+    ET.tostring function will print XML namespaces if the input XML has
+    an affiliated namespace.  This is presently necessary from the
+    repeated use of the DFXML sax code, but might not be necessary in
+    the future.
+    ElementTree-and-namespace references:
+      Reading with XML namespaces:
+        http://effbot.org/zone/element-namespaces.htm
+        http://stackoverflow.com/a/13475333/1207160
+        http://stackoverflow.com/a/1319417/1207160
+        http://stackoverflow.com/a/14853417/1207160
+      Writing with XML namespaces:
+        http://stackoverflow.com/a/3895958/1207160
+"""
+    import io
+    import xml.etree.ElementTree as ET
+
+    ET.register_namespace("", XMLNS_DFXML)
+
+    if not xmlfile:
+        raise ValueError("xmlfile must be specified")
+    qtagname = "{%s}fileobject" % XMLNS_DFXML
+    for event, elem in ET.iterparse(xmlfile, ("start","end")):
+        if event == "end":
+            #Note that ElementTree qualifies tag names if possible.  Thus, the paired check.
+            if elem.tag in ["fileobject", qtagname]:
+                xmlstring = ET.tostring(elem)
+                pseudof = io.BytesIO()
+                pseudof.write(xmlstring)
+                pseudof.seek(0)
+                def temp_callback(fi):
+                    #TODO The volumeobject isn't populated this way; need to catch with iterparse.
+                    if preserve_elements:
+                        fi.xml_element = elem
+                reader = read_dfxml(xmlfile=pseudof, imagefile=imagefile, callback=temp_callback, preserve_fis=True)
+                yield reader.fi_history[0]
+                if not preserve_elements:
+                    elem.clear()
+
+
+#This regular expression removes xmlns declarations from elements.
+#Note it will fail if the namespace includes a quote character.
+rx_xmlns = r"""\sxmlns(:\S+)?=['"][^'"]+['"]"""
+
+def ET_tostring(*pargs, **kwargs):
+    """
+    The ElementTree XML interface produces redundant namespace
+    declarations if you print an element at a time.  This method simply
+    removes all xmlns delcarations from the string.
+    """
+    global rx_xmlns
+    import xml.etree.ElementTree as ET
+    tempstring = ET.tostring(*pargs, **kwargs)
+    retval = re.sub(rx_xmlns, "", tempstring)
+    return retval
+
+
 def read_regxml(xmlfile=None,flags=0,callback=None):
     """Processes an image using expat, calling a callback for node encountered."""
     import xml.parsers.expat
@@ -1372,12 +1491,12 @@ def read_regxml(xmlfile=None,flags=0,callback=None):
     r = regxml_reader(flags=flags)
     try:
         r.process_xml_stream(xmlfile,callback)
-    except xml.parsers.expat.ExpatError:
+    except xml.parsers.expat.ExpatError as e:
         stderr.write("XML parsing error for file \"" + xmlfile.name + "\".  Object stack:\n")
         for x in r.objectstack:
             stderr.write(str(x) + "\n")
         stderr.write("(Done.)\n")
-        raise
+        raise e
     return r
 
 def fileobjects_sax(xmlfile=None,imagefile=None,flags=0):
@@ -1419,7 +1538,14 @@ def volumeobjects_sax(xmlfile=None,imagefile=None,flags=0):
     r.process_xml_stream(xmlfile,imagefile=None,callback=lambda vo:ret.append(vo))
     return ret
     
-        
+def creatorobjects_sax(xmlfile=None,flags=0):
+    r = creatorobject_reader()
+    ret = []
+    try:
+        r.process_xml_stream(xmlfile,callback=lambda vo:ret.append(vo))
+    except FinishedReadingCreator as e:
+        pass
+    return ret
         
 ################################################################
 if __name__=="__main__":
@@ -1512,3 +1638,8 @@ if __name__=="__main__":
         assert db.intersects(byte_run(-1,1))==None
         assert db.intersects(byte_run(10,1))==None
         print("Overlap engine good!")
+        assert re.sub(rx_xmlns, "", """<fileobject xmlns="http://www.forensicswiki.org/wiki/Category:Digital_Forensics_XML">""") == "<fileobject>"
+        assert re.sub(rx_xmlns, "", """<fileobject xmlns:dfxml="http://www.forensicswiki.org/wiki/Category:Digital_Forensics_XML">""") == "<fileobject>"
+        assert re.sub(rx_xmlns, "", """<fileobject delta:new_file="1">""") == """<fileobject delta:new_file="1">"""
+        assert re.sub(rx_xmlns, "", """<fileobject xmlns="http://www.forensicswiki.org/wiki/Category:Digital_Forensics_XML" attr="1">""") == """<fileobject attr="1">"""
+        print("XML namespace regex good!")
